package rice.post.storage;

import java.security.*;
import java.io.*;

import rice.p2p.commonapi.*;
import rice.p2p.past.*;

/**
* This class serves as a reference to a PostObject
 * stored in the Post system.  This class knows both the
 * location in the network and the encryption key of the
 * corresponding PostData object.
 * 
 * @version $Id: SignedReference.java 2172 2005-02-05 00:03:08Z amislove $
 */
public class SignedReference implements Serializable {
  
  // serialver for backwards compatibility
  private static final long serialVersionUID = -6793773589264272354L;
  
  /**
   * Location where this data is stored in PAST.
   */
  private Id location;

  /**
   * Contructs a PostDataReference object given
   * the address of the object.
   *
   * @param location The location in PAST of the PostData object
   */
  public SignedReference(Id location) {
    this.location = location;
  }

  /**
   * @return The location of the data referenced by this object
   */
  public Id getLocation() {
    return location;
  }
  
  public int hashCode() {
    return location.hashCode();
  }

  public boolean equals(Object o) {
    if (! (o instanceof SignedReference))
      return false;

    SignedReference ref = (SignedReference) o;

    return location.equals(ref.getLocation());
  }

  public String toString() {
    return "[SignedRef " + location + "]";
  }
}
